/*
*	Copyright (C) 2015
*	Chair of Process Control Engineering,
*	Aachen University of Technology.
*	All rights reserved.
*
*	Redistribution and use in source and binary forms, with or without
*	modification, are permitted provided that the following conditions
*	are met:
*	1. Redistributions of source code must retain the above copyright
*	   notice, this list of conditions and the following disclaimer.
*	2. Redistributions in binary form must print or display the above
*	   copyright notice either during startup or must have a means for
*	   the user to view the copyright notice.
*	3. Redistributions in binary form must reproduce the above copyright
*	   notice, this list of conditions and the following disclaimer in
*		the documentation and/or other materials provided with the
*		distribution.
*	4. Neither the name of the Chair of Process Control Engineering nor
*		the name of the Aachen University of Technology may be used to
*		endorse or promote products derived from this software without
*		specific prior written permission.
*
*	THIS SOFTWARE IS PROVIDED BY THE CHAIR OF PROCESS CONTROL ENGINEERING
*	``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE CHAIR OF
*	PROCESS CONTROL ENGINEERING BE LIABLE FOR ANY DIRECT, INDIRECT,
*	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
*	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
*	OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
*	AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
*	LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
*	WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*	POSSIBILITY OF SUCH DAMAGE.
*
***********************************************************************
*
*   FILE
*   ----
*   downloadApplication.c
*
*   History
*   -------
*   2013-04-29   File created
*
*******************************************************************************
*
*   This file is generated by the 'acplt_builder' command
*
******************************************************************************/


#ifndef OV_COMPILE_LIBRARY_cshmi
#define OV_COMPILE_LIBRARY_cshmi
#endif

#include "cshmilib.h"

//#define CSHMI_TIMING_DEBUG 1
//#define CSHMI_CHILDLIST_DEBUG 1

#ifdef CSHMI_TIMING_DEBUG
	OV_UINT functioncounter = 0;
	OV_UINT iterationcounter = 0;
#endif

/**
 * returns a list of ov_containment
 * embedment/OV_PART is known to the visualisation system
 * @param pObj object to start childlist
 * @param strResult Pointer to the result String
 * @param isFirstEntry if FALSE the function adds an "," as a separator
 * @return
 */
static OV_RESULT cshmi_downloadApplication_buildChildList(OV_INSTPTR_ov_domain pDom, OV_STRING *strResult, OV_BOOL isFirstEntry){
	OV_INSTPTR_ov_object pChildObj = NULL;
	OV_INSTPTR_cshmi_IfThenElse pIfThenElse = NULL;
	OV_INSTPTR_cshmi_ChildrenIterator pChildrenIterator = NULL;

	//local string to make ov_string functions fast
	OV_STRING strChildList = NULL;
	OV_RESULT result = OV_ERR_OK;

#ifdef CSHMI_TIMING_DEBUG
	functioncounter++;
#endif

	/* builds something like this (without the whitespaces):
	%22/TechUnits/cshmi%22:%7B
		%22ChildListParameters%22:%5B
			%22turbo /acplt/cshmi/downloadApplication%22,
			%22Templates /acplt/ov/domain%22
		%5D
	%7D,
	%22/TechUnits/cshmi/Templates%22:%7B
		%22ChildListParameters%22:%5B
		%5D
	%7D,
	%22/TechUnits/sheet%22:
	%7B
		%22ChildListParameters%22:%5B
		%5D
	%7D
	*/

#ifdef CSHMI_CHILDLIST_DEBUG
	ov_logfile_debug("Childlist: listing children for %s", pDom->v_identifier);
#endif


	if(isFirstEntry == FALSE){
		ov_string_setvalue(&strChildList, ",%22");
	}else{
		ov_string_setvalue(&strChildList, "%22");
	}
	ov_memstack_lock();
	ov_string_append(&strChildList, ov_path_getcanonicalpath(Ov_PtrUpCast(ov_object, pDom), 2));
	ov_memstack_unlock();
	ov_string_append(&strChildList, "%22:%7B");
	ov_string_append(&strChildList, "%22ChildListParameters%22:%5B");

	//iterate over all childrens and list them
	Ov_ForEachChild(ov_containment, pDom, pChildObj){
		ov_string_append(&strChildList, "%22");
		ov_string_append(&strChildList, pChildObj->v_identifier);
		ov_string_append(&strChildList, " ");
		ov_memstack_lock();
		ov_string_append(&strChildList, ov_path_getcanonicalpath(Ov_PtrUpCast(ov_object, Ov_GetClassPtr(pChildObj)), 2));
		ov_memstack_unlock();
		ov_string_append(&strChildList, "%22");
		if(Ov_GetNextChild(ov_containment, pChildObj) != NULL){
			ov_string_append(&strChildList, ",");
		}
	};
	ov_string_append(&strChildList, "%5D%7D");

	//one single append to the huge string
	result = ov_string_append(strResult, strChildList);
	ov_string_setvalue(&strChildList, NULL);
	if(Ov_Fail(result)){
		return result;
	}

	//some of our children should list their children, too
	Ov_ForEachChild(ov_containment, pDom, pChildObj){
		//interesting children are domains or some cshmi objects
		if(	Ov_GetClassPtr(pChildObj) == pclass_ov_domain
			|| Ov_CanCastTo(cshmi_ContainerElement, pChildObj)
			|| Ov_CanCastTo(cshmi_Element, pChildObj)
			|| Ov_GetClassPtr(pChildObj) == pclass_cshmi_SetConcatValue
			|| Ov_GetClassPtr(pChildObj) == pclass_cshmi_SetMathValue
			|| Ov_CanCastTo(cshmi_Event, pChildObj)
			){
			result = cshmi_downloadApplication_buildChildList(Ov_StaticPtrCast(ov_domain, pChildObj), strResult, FALSE);
			if(Ov_Fail(result)){
				return result;
			}
		}else
		//some children have no containment but their parts have interesting containments
		if(Ov_GetClassPtr(pChildObj) == pclass_cshmi_IfThenElse){
			pIfThenElse = Ov_StaticPtrCast(cshmi_IfThenElse, pChildObj);
			result = cshmi_downloadApplication_buildChildList(Ov_PtrUpCast(ov_domain, Ov_GetPartPtr(if, pIfThenElse)), strResult, FALSE);
			if(Ov_Fail(result)){
				return result;
			}
			result = cshmi_downloadApplication_buildChildList(Ov_PtrUpCast(ov_domain, Ov_GetPartPtr(then, pIfThenElse)), strResult, FALSE);
			if(Ov_Fail(result)){
				return result;
			}
			result = cshmi_downloadApplication_buildChildList(Ov_PtrUpCast(ov_domain, Ov_GetPartPtr(else, pIfThenElse)), strResult, FALSE);
			if(Ov_Fail(result)){
				return result;
			}
		}else if(Ov_GetClassPtr(pChildObj) == pclass_cshmi_ChildrenIterator){
			pChildrenIterator = Ov_StaticPtrCast(cshmi_ChildrenIterator, pChildObj);
			result = cshmi_downloadApplication_buildChildList(Ov_PtrUpCast(ov_domain, Ov_GetPartPtr(forEachChild, pChildrenIterator)), strResult, FALSE);
			if(Ov_Fail(result)){
				return result;
			}
#ifdef CSHMI_CHILDLIST_DEBUG
		}else{
			ov_logfile_debug("Childlist: skipped childrenlist for %s", pChildObj->v_identifier);
#endif
		}
	}
	return OV_ERR_OK;
}

#ifdef CSHMI_TIMING_DEBUG
#define INC_ITERATIONCOUNTER iterationcounter++
#else
#define INC_ITERATIONCOUNTER
#endif

#define CSHMI_BUILDSTRINGCLASS(classname)	\
	if(Ov_GetFirstChild(ov_instantiation, pclass_cshmi_##classname) != NULL){	\
		if(pDownloadApplication->v_ApplicationCache.cache##classname##Dirty == TRUE){	\
			Ov_ForEachChild(ov_instantiation, pclass_cshmi_##classname, pObj){	\
				INC_ITERATIONCOUNTER;	\
				/*	get config from this instance	*/	\
				ov_string_setvalue(&strIterate, cshmi_Object_getConfigAsJSON(Ov_StaticPtrCast(cshmi_Object, pObj))); /* from heap mem */	\
				if(strIterate && Ov_GetNextChild(ov_instantiation, pObj) != NULL){	\
					ov_string_append(&strIterate, ",");	\
				}	\
				/*	use this iteration result (append is NULL String safe)	*/	\
				ov_string_append(&ResultListVec[ResultListIndex], strIterate);	\
			}	\
			/*	remember class result in the heap	*/	\
			pDownloadApplication->v_ApplicationCache.cache##classname##Dirty = FALSE;	\
			strLen = ov_string_getlength(ResultListVec[ResultListIndex]);	\
			if(strLen != 0){	\
				pDownloadApplication->v_ApplicationCache.str##classname = Ov_HeapMalloc(strLen+1);	\
				if(pDownloadApplication->v_ApplicationCache.str##classname != NULL){	\
					memcpy(pDownloadApplication->v_ApplicationCache.str##classname, ResultListVec[ResultListIndex], strLen+1);	\
				}	\
			}else{	\
				Ov_HeapFree(pDownloadApplication->v_ApplicationCache.str##classname);	\
				pDownloadApplication->v_ApplicationCache.str##classname = NULL;	\
			}	\
		}else{	\
			ov_string_setvalue(&ResultListVec[ResultListIndex], pDownloadApplication->v_ApplicationCache.str##classname);	\
		}	\
		if(ResultListVec[ResultListIndex] != NULL){	\
			ResultListIndex++;	\
			if(elementIsfirst == TRUE){	\
				ov_string_append(&ResultFormat, "%s");	\
			}else{	\
				ov_string_append(&ResultFormat, ",%s");	\
			}	\
			elementIsfirst = FALSE;	\
		}	\
	}

/**
 *	builds the graphical elements into the cache
 * @param strResult
 * @return
 */
static OV_RESULT cshmi_downloadApplication_buildElementList(
		OV_INSTPTR_cshmi_downloadApplication        pDownloadApplication,
		OV_STRING	*strResult
){
	OV_INSTPTR_ov_object pObj = NULL;

	//be careful to adjust the ov_string_print at the end of the function
	#define MAXELEMENTENTRIES 11
	OV_STRING ResultListVec[MAXELEMENTENTRIES] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
	//Note: we do not precache blackbox or image! To risky with the encoding of the Strings.
	OV_STRING strIterate = NULL;
	OV_UINT strLen = 0;
	OV_UINT ResultListIndex = 0;
	OV_STRING ResultFormat = NULL;
	OV_BOOL elementIsfirst = TRUE;

	OV_STRING temp = NULL;
	OV_UINT i = 0;
	OV_RESULT result = OV_ERR_OK;
	ov_string_setvalue(&ResultFormat, "%s");

#ifdef CSHMI_TIMING_DEBUG
	functioncounter++;
#endif

	/* builds something like this (without the whitespaces):
	%22/TechUnits/sheet%22:%7B
		%22Parameters%22:%7B
			%22x%22:%220.000000%22,
			%22y%22:%220.000000%22,
			%22width%22:%221000.000000%22,
			%22height%22:%22900.000000%22,
			%22opacity%22:%221.000000%22,
			%22rotate%22:%220%22,
			%22hideable%22:%22FALSE%22,
			%22visible%22:%22TRUE%22,
			%22TemplateDefinition%22:%22%22,
			%22FBReference%22:%22%22,
			%22FBVariableReference%22:%22%22,
			%22ConfigValues%22:%22%22
		%7D
	%7D
	 */


	CSHMI_BUILDSTRINGCLASS(Group);
	CSHMI_BUILDSTRINGCLASS(TemplateDefinition);
	CSHMI_BUILDSTRINGCLASS(InstantiateTemplate);
	CSHMI_BUILDSTRINGCLASS(Rectangle);
	CSHMI_BUILDSTRINGCLASS(Circle);
	CSHMI_BUILDSTRINGCLASS(Text);
	CSHMI_BUILDSTRINGCLASS(Line);
	CSHMI_BUILDSTRINGCLASS(Polyline);
	CSHMI_BUILDSTRINGCLASS(Polygon);
	CSHMI_BUILDSTRINGCLASS(Path);
	CSHMI_BUILDSTRINGCLASS(Ellipse);

	//concat the result, in one batch for performance reasons (append is not cheap)
	result = ov_string_print(strResult, ResultFormat, *strResult, ResultListVec[0], ResultListVec[1], ResultListVec[2], ResultListVec[3], ResultListVec[4], ResultListVec[5], ResultListVec[6], ResultListVec[7], ResultListVec[8], ResultListVec[9], ResultListVec[10]);
	ov_string_setvalue(&ResultFormat, NULL);

	ov_string_setvalue(&temp, NULL);

	ov_string_setvalue(&strIterate, NULL);
	for(i = 0;i < MAXELEMENTENTRIES;i++){
		ov_string_setvalue(&ResultListVec[i], NULL);
	}

	return result;
}

//todo automagic determination if cache is used???


/**
 *	builds the interactive elements (events/actions/conditions)
 * @param strResult
 * @return
 */
static OV_RESULT cshmi_downloadApplication_buildActionList(
		OV_INSTPTR_cshmi_downloadApplication        pDownloadApplication,
		OV_STRING	*strResult
){
	OV_INSTPTR_ov_object pObj = NULL;

	//be careful to adjust the ov_string_print at the end of the function
	#define MAXACTIONENTRIES 13
	OV_STRING ResultListVec[MAXACTIONENTRIES] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
	OV_STRING strIterate = NULL;
	OV_UINT strLen = 0;
	OV_UINT ResultListIndex = 0;
	OV_STRING ResultFormat = NULL;
	OV_BOOL elementIsfirst = TRUE;
	OV_STRING ParameterName = NULL;
	OV_STRING ParameterValue = NULL;
	OV_STRING temp = NULL;
	OV_UINT i = 0;
	OV_RESULT result = OV_ERR_OK;
	ov_string_setvalue(&ResultFormat, "%s");

#ifdef CSHMI_TIMING_DEBUG
	functioncounter++;
#endif
	/* builds something like this (without the whitespaces):
	%22/TechUnits/sheet/onload/set%22:%7B
		%22translationSource%22:%22%22,
		%22ParameterName%22:%22elemVar%22,
		%22ParameterValue%22:%22fill%22,
		%22translationSource%22:%22%22
	%7D,
	%22/TechUnits/sheet/onload/set.value%22:%7B
		%22ParameterName%22:%22value%22,
		%22ParameterValue%22:%22red%22%
	%7D
	%22/TechUnits/sheet/onload/if.if/conditionA%22:%7B
		%22Parameters%22:%7B
			%22comptype%22:%22==%22,
			%22ignoreError%22:%22FALSE%22
		%7D
	%7D
	*/

/*
	OV_VTBLPTR_cshmi_Object	pVTableObj;
	OV_STRING HeapConfigAsJSON = NULL;

    Ov_GetVTablePtr(cshmi_Object, pVTableObj, pObj);
	if(!pVTableObj) {
		HeapConfigAsJSON = pVTableObj->m_getConfigAsJSON(pObj);
	}
	*/

	CSHMI_BUILDSTRINGCLASS(SetValue);
	CSHMI_BUILDSTRINGCLASS(SetConcatValue);
	CSHMI_BUILDSTRINGCLASS(SetMathValue);
	CSHMI_BUILDSTRINGCLASS(GetValue);
	CSHMI_BUILDSTRINGCLASS(ChildrenIterator);
	CSHMI_BUILDSTRINGCLASS(IfThenElse);
	CSHMI_BUILDSTRINGCLASS(Compare);
	CSHMI_BUILDSTRINGCLASS(CompareIteratedChild);
	CSHMI_BUILDSTRINGCLASS(TimeEvent);
	CSHMI_BUILDSTRINGCLASS(RoutePolyline);
	CSHMI_BUILDSTRINGCLASS(TranslationSource);
	CSHMI_BUILDSTRINGCLASS(CreateObject);
	CSHMI_BUILDSTRINGCLASS(Vibrate);


	//concat the result, manual for performance reasons (append is not cheap)
	result = ov_string_print(strResult, ResultFormat, *strResult, ResultListVec[0], ResultListVec[1], ResultListVec[2], ResultListVec[3], ResultListVec[4], ResultListVec[5], ResultListVec[6], ResultListVec[7], ResultListVec[8], ResultListVec[9], ResultListVec[10], ResultListVec[11], ResultListVec[12]);
	ov_string_setvalue(&ResultFormat, NULL);
	ov_string_setvalue(&ParameterName, NULL);
	ov_string_setvalue(&ParameterValue, NULL);
	ov_string_setvalue(&temp, NULL);

	ov_string_setvalue(&strIterate, NULL);
	for(i = 0;i < MAXACTIONENTRIES;i++){
		ov_string_setvalue(&ResultListVec[i], NULL);
	}

	return result;
}


/**
 * Tools:
 * http://meyerweb.com/eric/tools/dencoder/
 * http://jsonviewer.stack.hu/
 * http://jsonlint.com/
 */

/**
 *
 */
OV_DLLFNCEXPORT OV_STRING cshmi_downloadApplication_asJSON_get(
		OV_INSTPTR_cshmi_downloadApplication        pDownloadApplication
) {
	OV_INSTPTR_ov_domain pTUcshmi = NULL;
	OV_INSTPTR_cshmi_Group pGroup = NULL;
	OV_INSTPTR_ov_domain pGroupParent = NULL;
	OV_STRING returnString = NULL;
	OV_STRING strBaseKsPath = NULL;
	OV_STRING strElements = NULL;
	OV_STRING strActions = NULL;
	OV_STRING strChildList = NULL;
	OV_STRING pJSON = NULL;
	OV_UINT lenBaseKsPath = 0;
	OV_UINT lenElements = 0;
	OV_UINT lenActions = 0;
	OV_UINT lenChildList = 0;
	OV_RESULT result = OV_ERR_OK;

	#ifdef CSHMI_TIMING_DEBUG
		OV_UINT lenSubChildList = 0;
		OV_TIME lasttime;
		OV_TIME thistime;
		OV_TIME_SPAN diff;

		functioncounter = 0;
		iterationcounter = 0;
	#endif

	/**
	 * Umsetzungstabelle fr decodeURI
	 * " %22
	 * { %7B
	 * } %7D
	 * [ %5B
	 * ] %5D
	 * ( (
	 * ) )
	 * + +
	 */

	pTUcshmi = Ov_DynamicPtrCast(ov_domain, ov_path_getobjectpointer("/TechUnits/cshmi", 2));
	if(pTUcshmi == NULL){
		return (OV_STRING) NULL;
	}

	//todo add more or all
	//some static baseKsPath settings
	result = ov_string_setvalue(&strBaseKsPath, "%22baseKsPath%22:%7B%22/%22:%5B%5D,%22/TechUnits%22:%5B%5D,%22/TechUnits/cshmi%22:%5B%5D,%22/TechUnits/cshmi/Templates%22:%5B%5D%7D,");
	lenBaseKsPath = ov_string_getlength(strBaseKsPath);

	#ifdef CSHMI_TIMING_DEBUG
		ov_time_gettime(&lasttime);
	#endif

	//Elements is a list of configured elements
	ov_string_setvalue(&strElements, "%22Elements%22:%7B");
	result = cshmi_downloadApplication_buildElementList(pDownloadApplication, &strElements);
	if(Ov_Fail(result)){
		ov_logfile_debug("%d:%s Error building Elementlist: %s", __LINE__, __FILE__, ov_result_getresulttext(result));
		ov_string_setvalue(&strBaseKsPath, NULL);
		ov_string_setvalue(&strElements, NULL);
		ov_string_setvalue(&strActions, NULL);
		ov_string_setvalue(&strChildList, NULL);
		return (OV_STRING) 0;
	}else{
		ov_string_append(&strElements, "%7D,");
	}
	lenElements = ov_string_getlength(strElements);

	#ifdef CSHMI_TIMING_DEBUG
		ov_time_gettime(&thistime);
		ov_time_diff(&diff, &thistime, &lasttime);
		ov_logfile_debug("Time: %s", ov_time_timetoascii_local(&thistime));
		ov_logfile_debug("Elements    : %s (%u Bytes, %u function calls, %u iterations)", ov_time_timespantoascii(&diff), lenElements, functioncounter, iterationcounter);
		ov_time_gettime(&lasttime);
		functioncounter = 0;
		iterationcounter = 0;
	#endif

	//Actions is a list of events/actions/conditions, set/get stores the result only
	ov_string_setvalue(&strActions, "%22Actions%22:%7B");
	result = cshmi_downloadApplication_buildActionList(pDownloadApplication, &strActions);

	if(Ov_Fail(result)){
		ov_logfile_debug("%d:%s Error building Actionlist: %s", __LINE__, __FILE__, ov_result_getresulttext(result));
		ov_string_setvalue(&strBaseKsPath, NULL);
		ov_string_setvalue(&strElements, NULL);
		ov_string_setvalue(&strActions, NULL);
		ov_string_setvalue(&strChildList, NULL);
		return (OV_STRING) 0;
	}else{
		ov_string_append(&strActions, "%7D,");
	}
	lenActions = ov_string_getlength(strActions);

	#ifdef CSHMI_TIMING_DEBUG
		ov_time_gettime(&thistime);
		ov_time_diff(&diff, &thistime, &lasttime);
		ov_logfile_debug("Actions     : %s (%u Bytes, %u function calls, %u iterations)", ov_time_timespantoascii(&diff), lenActions, functioncounter, iterationcounter);
		ov_time_gettime(&lasttime);
		functioncounter = 0;
		iterationcounter = 0;
	#endif

	if(pDownloadApplication->v_ApplicationCache.cacheChildlistDirty){
		//ChildList is a list of ov_containment
		ov_string_setvalue(&strChildList, "%22ChildList%22:%7B");
		result = cshmi_downloadApplication_buildChildList(pTUcshmi, &strChildList, TRUE);

		#ifdef CSHMI_TIMING_DEBUG
			lenSubChildList = ov_string_getlength(strChildList);

			ov_time_gettime(&thistime);
			ov_time_diff(&diff, &thistime, &lasttime);
			ov_logfile_debug("Childlist CS: %s (%u Bytes, %u function calls, %u iterations)", ov_time_timespantoascii(&diff), lenSubChildList, functioncounter, iterationcounter);
			ov_time_gettime(&lasttime);
			functioncounter = 0;
			iterationcounter = 0;
		#endif

		//todo 2mbyte mb scheint fr den engineering testcase zu klein. also vergrern
		//ndern in memstack, also heap
		if(Ov_Fail(result)){
			ov_logfile_debug("%d:%s Error building Childlist: %s", __LINE__, __FILE__, ov_result_getresulttext(result));
			ov_string_setvalue(&strBaseKsPath, NULL);
			ov_string_setvalue(&strElements, NULL);
			ov_string_setvalue(&strActions, NULL);
			ov_string_setvalue(&strChildList, NULL);
			return (OV_STRING) 0;
		}
		//check for other groups (with no cshmi parent) outside of /TechUnits/cshmi
		Ov_ForEachChildEx(ov_instantiation, pclass_cshmi_Group, pGroup, cshmi_Group){
			pGroupParent = Ov_GetParent(ov_containment, pGroup);
			if(!Ov_CanCastTo(cshmi_Object, pGroupParent) && pGroupParent != pTUcshmi){
				result = cshmi_downloadApplication_buildChildList(Ov_PtrUpCast(ov_domain, pGroup), &strChildList, FALSE);
				if(Ov_Fail(result)){
					ov_logfile_debug("%d:%s Error building Childlist: %s", __LINE__, __FILE__, ov_result_getresulttext(result));
					ov_string_setvalue(&strBaseKsPath, NULL);
					ov_string_setvalue(&strElements, NULL);
					ov_string_setvalue(&strActions, NULL);
					ov_string_setvalue(&strChildList, NULL);
					return (OV_STRING) 0;
				}
			}
		}
		ov_string_append(&strChildList, "%7D");

		lenChildList = ov_string_getlength(strChildList);
		#ifdef CSHMI_TIMING_DEBUG
			ov_time_gettime(&thistime);
			ov_time_diff(&diff, &thistime, &lasttime);
			ov_logfile_debug("Childlist TU: %s (%u Bytes, %u function calls, %u iterations)", ov_time_timespantoascii(&diff), lenChildList-lenSubChildList, functioncounter, iterationcounter);
			ov_time_gettime(&lasttime);
			functioncounter = 0;
			iterationcounter = 0;
		#endif

		//cache this work in heap
		pDownloadApplication->v_ApplicationCache.strChildlist = (OV_STRING) Ov_HeapMalloc(lenChildList+1);
		if(!pDownloadApplication->v_ApplicationCache.strChildlist){
			ov_logfile_debug("%d:%s Error reserving memory for Childlist cache: heap full", __LINE__, __FILE__);
			ov_string_setvalue(&strBaseKsPath, NULL);
			ov_string_setvalue(&strActions, NULL);
			ov_string_setvalue(&strChildList, NULL);
			ov_string_setvalue(&strElements, NULL);
			return (OV_STRING) 0;
		}
		memcpy(pDownloadApplication->v_ApplicationCache.strChildlist, strChildList, lenChildList+1);
		pDownloadApplication->v_ApplicationCache.cacheChildlistDirty = FALSE;
		ov_string_setvalue(&strChildList, NULL);
	}else{
		//the length is required for the concat
		lenChildList = ov_string_getlength(pDownloadApplication->v_ApplicationCache.strChildlist);
		#ifdef CSHMI_TIMING_DEBUG
			ov_logfile_debug("Childlist   : 0000:00:00.000000 (%u Bytes, %u function calls, %u iterations)", lenChildList, functioncounter, iterationcounter);
		#endif
	}

	//we need space for all 4 strings, two times 3 Bytes and a null byte terminator
	pJSON = (OV_STRING) ov_memstack_alloc(lenChildList+lenBaseKsPath+lenElements+lenActions+7);
	if (!pJSON){
		ov_logfile_debug("%d:%s Error reserving memory for concating result: MEMSTACK full", __LINE__, __FILE__);
		ov_string_setvalue(&strBaseKsPath, NULL);
		ov_string_setvalue(&strActions, NULL);
		ov_string_setvalue(&strElements, NULL);
		return (OV_STRING) 0;
	}

	returnString = pJSON;
	//open JSON Element, concat Strings and close JSON Element
	memcpy(pJSON, "%7B", 3);
	pJSON = pJSON + 3;
	memcpy(pJSON, strBaseKsPath, lenBaseKsPath);
	pJSON = pJSON + lenBaseKsPath;
	memcpy(pJSON, strElements, lenElements);
	pJSON = pJSON + lenElements;
	memcpy(pJSON, strActions, lenActions);
	pJSON = pJSON + lenActions;
	memcpy(pJSON, pDownloadApplication->v_ApplicationCache.strChildlist, lenChildList);
	pJSON = pJSON + lenChildList;
	memcpy(pJSON, "%7D", 4);

	ov_string_setvalue(&strBaseKsPath, NULL);
	ov_string_setvalue(&strActions, NULL);
	ov_string_setvalue(&strElements, NULL);

#ifdef CSHMI_TIMING_DEBUG
	ov_time_gettime(&thistime);
	ov_time_diff(&diff, &thistime, &lasttime);
	ov_logfile_debug("strconcat   : %s", ov_time_timespantoascii(&diff));
	ov_time_gettime(&lasttime);
#endif

	return returnString;
}


OV_DLLFNCEXPORT void cshmi_downloadApplication_startup(
		OV_INSTPTR_ov_object 	pobj
) {
	/*
	 *   local variables
	 */
	OV_INSTPTR_cshmi_downloadApplication pDownloadApplication = Ov_StaticPtrCast(cshmi_downloadApplication, pobj);


	/* do what the base class does first */
	ov_object_startup(pobj);

	CSHMI_INITCLASSCACHEENTRY(Group);
	CSHMI_INITCLASSCACHEENTRY(TemplateDefinition);
	CSHMI_INITCLASSCACHEENTRY(InstantiateTemplate);
	CSHMI_INITCLASSCACHEENTRY(Rectangle);
	CSHMI_INITCLASSCACHEENTRY(Circle);
	CSHMI_INITCLASSCACHEENTRY(Text);
	CSHMI_INITCLASSCACHEENTRY(Line);
	CSHMI_INITCLASSCACHEENTRY(Polyline);
	CSHMI_INITCLASSCACHEENTRY(Polygon);
	CSHMI_INITCLASSCACHEENTRY(Path);
	CSHMI_INITCLASSCACHEENTRY(Ellipse);

	CSHMI_INITCLASSCACHEENTRY(SetValue);
	CSHMI_INITCLASSCACHEENTRY(SetConcatValue);
	CSHMI_INITCLASSCACHEENTRY(SetMathValue);
	CSHMI_INITCLASSCACHEENTRY(GetValue);
	CSHMI_INITCLASSCACHEENTRY(ChildrenIterator);
	CSHMI_INITCLASSCACHEENTRY(IfThenElse);
	CSHMI_INITCLASSCACHEENTRY(Compare);
	CSHMI_INITCLASSCACHEENTRY(CompareIteratedChild);
	CSHMI_INITCLASSCACHEENTRY(TimeEvent);
	CSHMI_INITCLASSCACHEENTRY(RoutePolyline);
	CSHMI_INITCLASSCACHEENTRY(TranslationSource);
	CSHMI_INITCLASSCACHEENTRY(CreateObject);
	CSHMI_INITCLASSCACHEENTRY(Vibrate);

	CSHMI_INITCLASSCACHEENTRY(Childlist);

	return;
}

OV_DLLFNCEXPORT void cshmi_downloadApplication_shutdown(
		OV_INSTPTR_ov_object 	pobj
) {
	/*
	 *   local variables
	 */
	OV_INSTPTR_cshmi_downloadApplication pDownloadApplication = Ov_StaticPtrCast(cshmi_downloadApplication, pobj);

	CSHMI_EMPTYCLASSCACHEENTRY(Group);
	CSHMI_EMPTYCLASSCACHEENTRY(TemplateDefinition);
	CSHMI_EMPTYCLASSCACHEENTRY(InstantiateTemplate);
	CSHMI_EMPTYCLASSCACHEENTRY(Rectangle);
	CSHMI_EMPTYCLASSCACHEENTRY(Circle);
	CSHMI_EMPTYCLASSCACHEENTRY(Text);
	CSHMI_EMPTYCLASSCACHEENTRY(Line);
	CSHMI_EMPTYCLASSCACHEENTRY(Polyline);
	CSHMI_EMPTYCLASSCACHEENTRY(Polygon);
	CSHMI_EMPTYCLASSCACHEENTRY(Path);
	CSHMI_EMPTYCLASSCACHEENTRY(Ellipse);

	CSHMI_EMPTYCLASSCACHEENTRY(SetValue);
	CSHMI_EMPTYCLASSCACHEENTRY(SetConcatValue);
	CSHMI_EMPTYCLASSCACHEENTRY(SetMathValue);
	CSHMI_EMPTYCLASSCACHEENTRY(GetValue);
	CSHMI_EMPTYCLASSCACHEENTRY(ChildrenIterator);
	CSHMI_EMPTYCLASSCACHEENTRY(IfThenElse);
	CSHMI_EMPTYCLASSCACHEENTRY(Compare);
	CSHMI_EMPTYCLASSCACHEENTRY(CompareIteratedChild);
	CSHMI_EMPTYCLASSCACHEENTRY(TimeEvent);
	CSHMI_EMPTYCLASSCACHEENTRY(RoutePolyline);
	CSHMI_EMPTYCLASSCACHEENTRY(TranslationSource);
	CSHMI_EMPTYCLASSCACHEENTRY(CreateObject);
	CSHMI_EMPTYCLASSCACHEENTRY(Vibrate);

	CSHMI_EMPTYCLASSCACHEENTRY(Childlist);

	/* set the object's state to "shut down" */
	ov_object_shutdown(pobj);

	return;
}
